# test spaces in hierarchy
# test tabs in hierarchy
# test mixed
import os

from django.conf import settings
from django.test import TestCase, override_settings
from categories.models import Category
from categories.management.commands.import_categories import Command
from django.core.management.base import CommandError


@override_settings(INSTALLED_APPS=(app for app in settings.INSTALLED_APPS if app != 'django.contrib.flatpages'))
class CategoryImportTest(TestCase):
    def setUp(self):
        pass

    def _import_file(self, filename):
        root_cats = ['Category 1', 'Category 2', 'Category 3']
        testfile = os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'fixtures', filename))
        cmd = Command()
        cmd.handle(testfile)
        roots = Category.tree.root_nodes()

        self.assertEqual(len(roots), 3)
        for item in roots:
            assert item.name in root_cats

        cat2 = Category.objects.get(name='Category 2')
        cat21 = cat2.children.all()[0]
        self.assertEqual(cat21.name, 'Category 2-1')
        cat211 = cat21.children.all()[0]
        self.assertEqual(cat211.name, 'Category 2-1-1')

    def testImportSpaceDelimited(self):
        Category.objects.all().delete()
        self._import_file('test_category_spaces.txt')

        items = Category.objects.all()

        self.assertEqual(items[0].name, 'Category 1')
        self.assertEqual(items[1].name, 'Category 1-1')
        self.assertEqual(items[2].name, 'Category 1-2')

    def testImportTabDelimited(self):
        Category.objects.all().delete()
        self._import_file('test_category_tabs.txt')

        items = Category.objects.all()

        self.assertEqual(items[0].name, 'Category 1')
        self.assertEqual(items[1].name, 'Category 1-1')
        self.assertEqual(items[2].name, 'Category 1-2')

    def testMixingTabsSpaces(self):
        """
        Should raise an exception.
        """
        string1 = ["cat1", "    cat1-1", "\tcat1-2-FAIL!", ""]
        string2 = ["cat1", "\tcat1-1", "    cat1-2-FAIL!", ""]
        cmd = Command()

        # raise Exception
        self.assertRaises(CommandError, cmd.parse_lines, string1)
        self.assertRaises(CommandError, cmd.parse_lines, string2)



from django.apps import AppConfig


class CategoriesConfig(AppConfig):
    name = 'categories'
    verbose_name = "Categories"

    def __init__(self, *args, **kwargs):
        super(CategoriesConfig, self).__init__(*args, **kwargs)
        from django.db.models.signals import class_prepared
        class_prepared.connect(handle_class_prepared)

    def ready(self):
        from django.db.models.signals import post_migrate
        from .migration import migrate_app

        post_migrate.connect(migrate_app)


def handle_class_prepared(sender, **kwargs):
    """
    See if this class needs registering of fields
    """
    from .settings import M2M_REGISTRY, FK_REGISTRY
    from .registration import registry
    sender_app = sender._meta.app_label
    sender_name = sender._meta.model_name

    for key, val in list(FK_REGISTRY.items()):
        app_name, model_name = key.split('.')
        if app_name == sender_app and sender_name == model_name:
            registry.register_model(app_name, sender, 'ForeignKey', val)

    for key, val in list(M2M_REGISTRY.items()):
        app_name, model_name = key.split('.')
        if app_name == sender_app and sender_name == model_name:
            registry.register_model(app_name, sender, 'ManyToManyField', val)



# Test adding 1 fk string
# Test adding 1 fk dict
# test adding many-to-many
# test adding 1 fk, 1 m2m
import django
from django.test import TestCase

from categories.registration import _process_registry, registry


class CategoryRegistrationTest(TestCase):
    """
    Test various aspects of adding fields to a model.
    """

    def test_foreignkey_string(self):
        FK_REGISTRY = {
            'flatpages.flatpage': 'category'
        }
        _process_registry(FK_REGISTRY, registry.register_fk)
        from django.contrib.flatpages.models import FlatPage
        self.assertTrue('category' in [f.name for f in FlatPage()._meta.get_fields()])

    def test_foreignkey_dict(self):
        FK_REGISTRY = {
            'flatpages.flatpage': {'name': 'category'}
        }
        _process_registry(FK_REGISTRY, registry.register_fk)
        from django.contrib.flatpages.models import FlatPage
        self.assertTrue('category' in [f.name for f in FlatPage()._meta.get_fields()])

    def test_foreignkey_list(self):
        FK_REGISTRY = {
            'flatpages.flatpage': (
                {'name': 'category', 'related_name': 'cats'},
            )
        }
        _process_registry(FK_REGISTRY, registry.register_fk)
        from django.contrib.flatpages.models import FlatPage
        self.assertTrue('category' in [f.name for f in FlatPage()._meta.get_fields()])

    if django.VERSION[1] >= 7:
        def test_new_foreignkey_string(self):
            registry.register_model('flatpages', 'flatpage', 'ForeignKey', 'category')
            from django.contrib.flatpages.models import FlatPage
            self.assertTrue('category' in [f.name for f in FlatPage()._meta.get_fields()])


class Categorym2mTest(TestCase):
    def test_m2m_string(self):
        M2M_REGISTRY = {
            'flatpages.flatpage': 'categories'
        }
        _process_registry(M2M_REGISTRY, registry.register_m2m)
        from django.contrib.flatpages.models import FlatPage
        self.assertTrue('category' in [f.name for f in FlatPage()._meta.get_fields()])



from django.shortcuts import get_object_or_404
from django.http import HttpResponse, Http404
from django.template.loader import select_template
from django.utils.translation import ugettext_lazy as _
from django.views.generic import DetailView, ListView

from .models import Category


def category_detail(request, path, template_name='categories/category_detail.html', extra_context={}):
    path_items = path.strip('/').split('/')
    if len(path_items) >= 2:
        category = get_object_or_404(
            Category,
            slug__iexact=path_items[-1],
            level=len(path_items) - 1,
            parent__slug__iexact=path_items[-2])
    else:
        category = get_object_or_404(
            Category,
            slug__iexact=path_items[-1],
            level=len(path_items) - 1)

    templates = []
    while path_items:
        templates.append('categories/%s.html' % '_'.join(path_items))
        path_items.pop()
    templates.append(template_name)

    context = {'category': category}
    if extra_context:
        context.update(extra_context)
    return HttpResponse(select_template(templates).render(context))


def get_category_for_path(path, queryset=Category.objects.all()):
    path_items = path.strip('/').split('/')
    if len(path_items) >= 2:
        queryset = queryset.filter(
            slug__iexact=path_items[-1],
            level=len(path_items) - 1,
            parent__slug__iexact=path_items[-2])
    else:
        queryset = queryset.filter(
            slug__iexact=path_items[-1],
            level=len(path_items) - 1)
    return queryset.get()


class CategoryDetailView(DetailView):
    model = Category
    path_field = 'path'

    def get_object(self, **kwargs):
        if self.path_field not in self.kwargs:
            raise AttributeError("Category detail view %s must be called with "
                                 "a %s." % (self.__class__.__name__, self.path_field))
        if self.queryset is None:
            queryset = self.get_queryset()
        try:
            return get_category_for_path(self.kwargs[self.path_field], self.model.objects.all())
        except Category.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})

    def get_template_names(self):
        names = []
        path_items = self.kwargs[self.path_field].strip('/').split('/')
        while path_items:
            names.append('categories/%s.html' % '_'.join(path_items))
            path_items.pop()
        names.extend(super(CategoryDetailView, self).get_template_names())
        return names


class CategoryRelatedDetail(DetailView):
    path_field = 'category_path'
    object_name_field = None

    def get_object(self, **kwargs):
        if self.path_field not in self.kwargs:
            raise AttributeError("Category detail view %s must be called with "
                                 "a %s." % (self.__class__.__name__, self.path_field))
        queryset = super(CategoryRelatedDetail, self).get_queryset()
        try:
            category = get_category_for_path(self.kwargs[self.path_field])
        except Category.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return queryset.get(category=category)

    def get_template_names(self):
        names = []
        opts = self.object._meta
        path_items = self.kwargs[self.path_field].strip('/').split('/')
        if self.object_name_field:
            path_items.append(getattr(self.object, self.object_name_field))
        while path_items:
            names.append('%s/category_%s_%s%s.html' % (
                opts.app_label,
                '_'.join(path_items),
                opts.object_name.lower(),
                self.template_name_suffix)
            )
            path_items.pop()
        names.append('%s/category_%s%s.html' % (
            opts.app_label,
            opts.object_name.lower(),
            self.template_name_suffix)
        )
        names.extend(super(CategoryRelatedDetail, self).get_template_names())
        return names


class CategoryRelatedList(ListView):
    path_field = 'category_path'

    def get_queryset(self):
        if self.path_field not in self.kwargs:
            raise AttributeError("Category detail view %s must be called with "
                                 "a %s." % (self.__class__.__name__, self.path_field))
        queryset = super(CategoryRelatedList, self).get_queryset()
        category = get_category_for_path(self.kwargs[self.path_field])
        return queryset.filter(category=category)

    def get_template_names(self):
        names = []
        if hasattr(self.object_list, 'model'):
            opts = self.object_list.model._meta
            path_items = self.kwargs[self.path_field].strip('/').split('/')
            while path_items:
                names.append('%s/category_%s_%s%s.html' % (
                    opts.app_label,
                    '_'.join(path_items),
                    opts.object_name.lower(),
                    self.template_name_suffix)
                )
                path_items.pop()
            names.append('%s/category_%s%s.html' % (
                opts.app_label,
                opts.object_name.lower(),
                self.template_name_suffix)
            )
        names.extend(super(CategoryRelatedList, self).get_template_names())
        return names



from django import template
from django.apps import apps
from django.template import (Node, TemplateSyntaxError, VariableDoesNotExist)
from django.template.base import FilterExpression
from six import string_types
from categories.base import CategoryBase
from categories.models import Category
from mptt.utils import drilldown_tree_for_node
from mptt.templatetags.mptt_tags import (tree_path, tree_info, RecurseTreeNode,
                                         full_tree_for_model)

register = template.Library()

register.filter("category_path", tree_path)
register.filter(tree_info)
register.tag("full_tree_for_category", full_tree_for_model)


def resolve(var, context):
    try:
        return var.resolve(context)
    except VariableDoesNotExist:
        try:
            return var.var
        except AttributeError:
            return var


def get_cat_model(model):
    """
    Return a class from a string or class
    """
    try:
        if isinstance(model, string_types):
            model_class = apps.get_model(*model.split("."))
        elif issubclass(model, CategoryBase):
            model_class = model
        if model_class is None:
            raise TypeError
    except TypeError:
        raise TemplateSyntaxError("Unknown model submitted: %s" % model)
    return model_class


def get_category(category_string, model=Category):
    """
    Convert a string, including a path, and return the Category object
    """
    model_class = get_cat_model(model)
    category = str(category_string).strip("'\"")
    category = category.strip('/')

    cat_list = category.split('/')
    if len(cat_list) == 0:
        return None
    try:
        categories = model_class.objects.filter(name=cat_list[-1], level=len(cat_list) - 1)
        if len(cat_list) == 1 and len(categories) > 1:
            return None
        # If there is only one, use it. If there is more than one, check
        # if the parent matches the parent passed in the string
        if len(categories) == 1:
            return categories[0]
        else:
            for item in categories:
                if item.parent.name == cat_list[-2]:
                    return item
    except model_class.DoesNotExist:
        return None


class CategoryDrillDownNode(template.Node):
    def __init__(self, category, varname, model):
        self.category = category
        self.varname = varname
        self.model = model

    def render(self, context):
        category = resolve(self.category, context)
        if isinstance(category, CategoryBase):
            cat = category
        else:
            cat = get_category(category, self.model)
        try:
            if cat is not None:
                context[self.varname] = drilldown_tree_for_node(cat)
            else:
                context[self.varname] = []
        except Exception:
            context[self.varname] = []
        return ''


@register.tag
def get_category_drilldown(parser, token):
    """
    Retrieves the specified category, its ancestors and its immediate children
    as an iterable.

    Syntax::

        {% get_category_drilldown "category name" [using "app.Model"] as varname %}

    Example::

        {% get_category_drilldown "/Grandparent/Parent" [using "app.Model"] as family %}

    or ::

        {% get_category_drilldown category_obj as family %}

    Sets family to::

        Grandparent, Parent, Child 1, Child 2, Child n
    """
    bits = token.split_contents()
    error_str = '%(tagname)s tag should be in the format {%% %(tagname)s ' \
                '"category name" [using "app.Model"] as varname %%} or ' \
                '{%% %(tagname)s category_obj as varname %%}.'
    if len(bits) == 4:
        if bits[2] != 'as':
            raise template.TemplateSyntaxError(error_str % {'tagname': bits[0]})
        if bits[2] == 'as':
            varname = bits[3].strip("'\"")
            model = "categories.category"
    if len(bits) == 6:
        if bits[2] not in ('using', 'as') or bits[4] not in ('using', 'as'):
            raise template.TemplateSyntaxError(error_str % {'tagname': bits[0]})
        if bits[2] == 'as':
            varname = bits[3].strip("'\"")
            model = bits[5].strip("'\"")
        if bits[2] == 'using':
            varname = bits[5].strip("'\"")
            model = bits[3].strip("'\"")
    category = FilterExpression(bits[1], parser)
    return CategoryDrillDownNode(category, varname, model)


@register.inclusion_tag('categories/breadcrumbs.html')
def breadcrumbs(category_string, separator=' > ', using='categories.category'):
    """
    {% breadcrumbs category separator="::" using="categories.category" %}

    Render breadcrumbs, using the ``categories/breadcrumbs.html`` template,
    using the optional ``separator`` argument.
    """
    cat = get_category(category_string, using)

    return {'category': cat, 'separator': separator}


@register.inclusion_tag('categories/ul_tree.html')
def display_drilldown_as_ul(category, using='categories.Category'):
    """
    Render the category with ancestors and children using the
    ``categories/ul_tree.html`` template.

    Example::

        {% display_drilldown_as_ul "/Grandparent/Parent" %}

    or ::

        {% display_drilldown_as_ul category_obj %}

    Returns::

        <ul>
          <li><a href="/categories/">Top</a>
          <ul>
            <li><a href="/categories/grandparent/">Grandparent</a>
            <ul>
              <li><a href="/categories/grandparent/parent/">Parent</a>
              <ul>
                <li><a href="/categories/grandparent/parent/child1">Child1</a></li>
                <li><a href="/categories/grandparent/parent/child2">Child2</a></li>
                <li><a href="/categories/grandparent/parent/child3">Child3</a></li>
              </ul>
              </li>
            </ul>
            </li>
          </ul>
          </li>
        </ul>
    """
    cat = get_category(category, using)
    if cat is None:
        return {'category': cat, 'path': []}
    else:
        return {'category': cat, 'path': drilldown_tree_for_node(cat)}


@register.inclusion_tag('categories/ul_tree.html')
def display_path_as_ul(category, using='categories.Category'):
    """
    Render the category with ancestors, but no children using the
    ``categories/ul_tree.html`` template.

    Example::

        {% display_path_as_ul "/Grandparent/Parent" %}

    or ::

        {% display_path_as_ul category_obj %}

    Returns::

        <ul>
            <li><a href="/categories/">Top</a>
            <ul>
                <li><a href="/categories/grandparent/">Grandparent</a></li>
            </ul>
            </li>
        </ul>
    """
    if isinstance(category, CategoryBase):
        cat = category
    else:
        cat = get_category(category)

    return {'category': cat, 'path': cat.get_ancestors() or []}


class TopLevelCategoriesNode(template.Node):
    def __init__(self, varname, model):
        self.varname = varname
        self.model = model

    def render(self, context):
        model = get_cat_model(self.model)
        context[self.varname] = model.objects.filter(parent=None).order_by('name')
        return ''


@register.tag
def get_top_level_categories(parser, token):
    """
    Retrieves an alphabetical list of all the categories that have no parents.

    Syntax::

        {% get_top_level_categories [using "app.Model"] as categories %}

    Returns an list of categories [<category>, <category>, <category, ...]
    """
    bits = token.split_contents()
    usage = 'Usage: {%% %s [using "app.Model"] as <variable> %%}' % bits[0]
    if len(bits) == 3:
        if bits[1] != 'as':
            raise template.TemplateSyntaxError(usage)
        varname = bits[2]
        model = "categories.category"
    elif len(bits) == 5:
        if bits[1] not in ('as', 'using') and bits[3] not in ('as', 'using'):
            raise template.TemplateSyntaxError(usage)
        if bits[1] == 'using':
            model = bits[2].strip("'\"")
            varname = bits[4].strip("'\"")
        else:
            model = bits[4].strip("'\"")
            varname = bits[2].strip("'\"")

    return TopLevelCategoriesNode(varname, model)


def get_latest_objects_by_category(category, app_label, model_name, set_name, date_field='pub_date', num=15):
    m = apps.get_model(app_label, model_name)
    if not isinstance(category, CategoryBase):
        category = Category.objects.get(slug=str(category))
    children = category.children.all()
    ids = []
    for cat in list(children) + [category]:
        if hasattr(cat, '%s_set' % set_name):
            ids.extend([x.pk for x in getattr(cat, '%s_set' % set_name).all()[:num]])

    return m.objects.filter(pk__in=ids).order_by('-%s' % date_field)[:num]


class LatestObjectsNode(Node):
    def __init__(self, var_name, category, app_label, model_name, set_name,
                 date_field='pub_date', num=15):
        """
        Get latest objects of app_label.model_name
        """
        self.category = category
        self.app_label = app_label
        self.model_name = model_name
        self.set_name = set_name
        self.date_field = date_field
        self.num = num
        self.var_name = var_name

    def render(self, context):
        """
        Render this sucker
        """
        category = resolve(self.category, context)
        app_label = resolve(self.app_label, context)
        model_name = resolve(self.model_name, context)
        set_name = resolve(self.set_name, context)
        date_field = resolve(self.date_field, context)
        num = resolve(self.num, context)

        result = get_latest_objects_by_category(category, app_label, model_name, set_name, date_field, num)
        context[self.var_name] = result

        return ''


def do_get_latest_objects_by_category(parser, token):
    """
    Get the latest objects by category

    {% get_latest_objects_by_category category app_name model_name set_name [date_field] [number] as [var_name] %}
    """
    proper_form = "{% get_latest_objects_by_category category app_name model_name set_name [date_field] [number] as [var_name] %}"
    bits = token.split_contents()

    if bits[-2] != 'as':
        raise TemplateSyntaxError("%s tag shoud be in the form: %s" % (bits[0], proper_form))
    if len(bits) < 7:
        raise TemplateSyntaxError("%s tag shoud be in the form: %s" % (bits[0], proper_form))
    if len(bits) > 9:
        raise TemplateSyntaxError("%s tag shoud be in the form: %s" % (bits[0], proper_form))
    category = FilterExpression(bits[1], parser)
    app_label = FilterExpression(bits[2], parser)
    model_name = FilterExpression(bits[3], parser)
    set_name = FilterExpression(bits[4], parser)
    var_name = bits[-1]
    if bits[5] != 'as':
        date_field = FilterExpression(bits[5], parser)
    else:
        date_field = FilterExpression(None, parser)
    if bits[6] != 'as':
        num = FilterExpression(bits[6], parser)
    else:
        num = FilterExpression(None, parser)
    return LatestObjectsNode(var_name, category, app_label, model_name, set_name, date_field, num)


register.tag("get_latest_objects_by_category", do_get_latest_objects_by_category)


@register.filter
def tree_queryset(value):
    """
    Converts a normal queryset from an MPTT model to include all the ancestors
    so a filtered subset of items can be formatted correctly
    """
    from django.db.models.query import QuerySet
    from copy import deepcopy
    if not isinstance(value, QuerySet):
        return value

    qs = value
    qs2 = deepcopy(qs)
    # Reaching into the bowels of query sets to find out whether the qs is
    # actually filtered and we need to do the INCLUDE_ANCESTORS dance at all.
    # INCLUDE_ANCESTORS is quite expensive, so don't do it if not needed.
    is_filtered = bool(qs.query.where.children)
    if is_filtered:
        include_pages = set()
        # Order by 'rght' will return the tree deepest nodes first;
        # this cuts down the number of queries considerably since all ancestors
        # will already be in include_pages when they are checked, thus not
        # trigger additional queries.
        for p in qs2.order_by('rght').iterator():
            if p.parent_id and p.parent_id not in include_pages and p.id not in include_pages:
                ancestor_id_list = p.get_ancestors().values_list('id', flat=True)
                include_pages.update(ancestor_id_list)

        if include_pages:
            qs = qs | qs.model._default_manager.filter(id__in=include_pages)

        qs = qs.distinct()
    return qs


@register.tag
def recursetree(parser, token):
    """
    Iterates over the nodes in the tree, and renders the contained block for each node.
    This tag will recursively render children into the template variable {{ children }}.
    Only one database query is required (children are cached for the whole tree)

    Usage:
            <ul>
                {% recursetree nodes %}
                    <li>
                        {{ node.name }}
                        {% if not node.is_leaf_node %}
                            <ul>
                                {{ children }}
                            </ul>
                        {% endif %}
                    </li>
                {% endrecursetree %}
            </ul>
    """
    bits = token.contents.split()
    if len(bits) != 2:
        raise template.TemplateSyntaxError('%s tag requires a queryset' % bits[0])
    queryset_var = FilterExpression(bits[1], parser)

    template_nodes = parser.parse(('endrecursetree',))
    parser.delete_first_token()

    return RecurseTreeNode(template_nodes, queryset_var)



from django.test import TestCase
from django import template
import re

from categories.models import Category


class CategoryTagsTest(TestCase):

    fixtures = ['musicgenres.json']

    def render_template(self, template_string, context={}):
        """
        Return the rendered string or raise an exception.
        """
        tpl = template.Template(template_string)
        ctxt = template.Context(context)
        return tpl.render(ctxt)

    def testTooFewArguments(self):
        """
        Ensure that get_category raises an exception if there aren't enough arguments.
        """
        self.assertRaises(template.TemplateSyntaxError, self.render_template, '{% load category_tags %}{% get_category %}')

    def testBasicUsage(self):
        """
        Test that we can properly retrieve a category.
        """
        # display_path_as_ul
        rock_resp = '<ul><li><a href="/categories/">Top</a></li></ul>'
        resp = self.render_template('{% load category_tags %}{% display_path_as_ul "/Rock" %}')
        resp = re.sub(r'\n$', "", resp)
        self.assertEqual(resp, rock_resp)

        # display_drilldown_as_ul
        expected_resp = '<ul><li><a href="/categories/">Top</a><ul><li><a href="/categories/world/">World</a><ul><li><strong>Worldbeat</strong><ul><li><a href="/categories/world/worldbeat/afrobeat/">Afrobeat</a></li></ul></li></ul></li></ul></li></ul>'
        resp = self.render_template(
            '{% load category_tags %}'
            '{% display_drilldown_as_ul "/World/Worldbeat" "categories.category" %}')
        resp = re.sub(r'\n$', "", resp)
        self.assertEqual(resp, expected_resp)

        # breadcrumbs
        expected_resp = '<a href="/categories/world/">World</a> &gt; Worldbeat'
        resp = self.render_template(
            '{% load category_tags %}'
            '{% breadcrumbs "/World/Worldbeat" " &gt; " "categories.category" %}')
        self.assertEqual(resp, expected_resp)

        # get_top_level_categories
        expected_resp = 'Avant-garde|Blues|Country|Easy listening|Electronic|Hip hop/Rap music|Jazz|Latin|Modern folk|Pop|Reggae|Rhythm and blues|Rock|World|'
        resp = self.render_template(
            '{% load category_tags %}'
            '{% get_top_level_categories using "categories.category" as varname %}'
            '{% for item in varname %}{{ item }}|{% endfor %}')
        self.assertEqual(resp, expected_resp)

        # get_category_drilldown
        expected_resp = "World|World &gt; Worldbeat|"
        resp = self.render_template(
            '{% load category_tags %}'
            '{% get_category_drilldown "/World" using "categories.category" as var %}'
            '{% for item in var %}{{ item }}|{% endfor %}')
        self.assertEqual(resp, expected_resp)

        # recursetree
        expected_resp = '<ul><li>Country<ul><li>Country pop<ul><li>Urban Cowboy</li></ul></li></ul></li><li>World<ul><li>Worldbeat<ul></ul></li></ul></li></ul>'
        ctxt = {'nodes': Category.objects.filter(name__in=("Worldbeat", "Urban Cowboy"))}
        resp = self.render_template(
            '{% load category_tags %}'
            '<ul>{% recursetree nodes|tree_queryset %}<li>{{ node.name }}'
            '{% if not node.is_leaf_node %}<ul>{{ children }}'
            '</ul>{% endif %}</li>{% endrecursetree %}</ul>', ctxt)
        self.assertEqual(resp, expected_resp)



from django.core.management.base import BaseCommand
from django.core.management.base import CommandError


class Command(BaseCommand):
    """
    Alter one or more models' tables with the registered attributes
    """
    help = "Drop the given field from the given model's table"
    args = "appname modelname fieldname"
    can_import_settings = True
    requires_system_checks = False

    def add_arguments(self, parser):
        parser.add_argument('app_name')
        parser.add_argument('model_name')
        parser.add_argument('field_name')

    def handle(self, *args, **options):
        """
        Alter the tables
        """
        from categories.migration import drop_field
        if 'app_name' not in options or 'model_name' not in options or 'field_name' not in options:
            raise CommandError("You must specify an Application name, a Model name and a Field name")

        drop_field(options['app_name'], options['model_name'], options['field_name'])



"""
Provides compatibility with Django 1.8
"""
from django.contrib.admin.utils import display_for_field as _display_for_field


def display_for_field(value, field, empty_value_display=None):
    try:
        return _display_for_field(value, field, empty_value_display)
    except TypeError:
        return _display_for_field(value, field)



# -*- coding: utf-8 -*-
from django.core import management
from django.core.management.base import CommandError
from django.db import connection
from django.test import TestCase


class TestMgmtCommands(TestCase):

    @classmethod
    def setUpClass(cls):
        connection.disable_constraint_checking()
        super(TestMgmtCommands, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(TestMgmtCommands, cls).tearDownClass()
        connection.enable_constraint_checking()

    def test_add_category_fields(self):
        management.call_command('add_category_fields', verbosity=0)

    def test_add_category_fields_app(self):
        management.call_command('add_category_fields', 'flatpages', verbosity=0)

    def test_drop_category_field(self):
        management.call_command('drop_category_field', 'flatpages', 'flatpage', 'category', verbosity=0)

    def test_drop_category_field_error(self):
        self.assertRaises(CommandError, management.call_command, 'drop_category_field', verbosity=0)



from django.conf import settings
import django

DJANGO10_COMPAT = django.VERSION[0] < 1 or (django.VERSION[0] == 1 and django.VERSION[1] < 1)

STATIC_URL = getattr(settings, 'STATIC_URL', settings.MEDIA_URL)
if STATIC_URL is None:
    STATIC_URL = settings.MEDIA_URL
MEDIA_PATH = getattr(settings, 'EDITOR_MEDIA_PATH', '%seditor/' % STATIC_URL)

TREE_INITIAL_STATE = getattr(settings, 'EDITOR_TREE_INITIAL_STATE', 'collapsed')

IS_GRAPPELLI_INSTALLED = 'grappelli' in settings.INSTALLED_APPS



from django.http import Http404
from django.contrib.auth.models import AnonymousUser
from django.test import Client, TestCase, RequestFactory
from categories.models import Category, CategoryRelation
from categories import views


class MyCategoryRelationView(views.CategoryRelatedDetail):
    model = CategoryRelation


class TestCategoryViews(TestCase):
    fixtures = ['musicgenres.json', ]

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

    def test_category_detail(self):
        cat0 = Category.objects.get(slug='country', level=0)
        cat1 = cat0.children.get(slug='country-pop')
        cat2 = Category.objects.get(slug='urban-cowboy')
        url = cat0.get_absolute_url()
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        url = cat1.get_absolute_url()
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        url = cat2.get_absolute_url()
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        response = self.client.get("%sfoo/" % url)
        self.assertEquals(response.status_code, 404)

    def test_get_category_for_path(self):
        cat0 = Category.objects.get(slug='country', level=0)
        cat1 = cat0.children.get(slug='country-pop')
        cat2 = Category.objects.get(slug='urban-cowboy')

        result = views.get_category_for_path('/country/country-pop/urban-cowboy/')
        self.assertEquals(result, cat2)
        result = views.get_category_for_path('/country/country-pop/')
        self.assertEquals(result, cat1)
        result = views.get_category_for_path('/country/')
        self.assertEquals(result, cat0)

    def test_categorydetailview(self):
        request = self.factory.get('')
        request.user = AnonymousUser()
        self.assertRaises(AttributeError, views.CategoryDetailView.as_view(), request)

        request = self.factory.get('')
        request.user = AnonymousUser()
        response = views.CategoryDetailView.as_view()(request, path='/country/country-pop/urban-cowboy/')
        self.assertEquals(response.status_code, 200)

        request = self.factory.get('')
        request.user = AnonymousUser()
        self.assertRaises(Http404, views.CategoryDetailView.as_view(), request, path='/country/country-pop/foo/')

    def test_categoryrelateddetailview(self):
        from simpletext.models import SimpleText
        stext = SimpleText.objects.create(
            name='Test',
            description='test description'
        )
        cat = Category.objects.get(slug='urban-cowboy')
        cat_rel = CategoryRelation.objects.create(  # NOQA
            category=cat,
            content_object=stext
        )
        request = self.factory.get('')
        request.user = AnonymousUser()
        self.assertRaises(AttributeError, MyCategoryRelationView.as_view(), request)

        request = self.factory.get('')
        request.user = AnonymousUser()
        response = MyCategoryRelationView.as_view()(request, category_path='/country/country-pop/urban-cowboy/')
        self.assertEquals(response.status_code, 200)

        request = self.factory.get('')
        request.user = AnonymousUser()
        self.assertRaises(Http404, MyCategoryRelationView.as_view(), request, category_path='/country/country-pop/foo/')



from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse, NoReverseMatch
import json


class GenericCollectionInlineModelAdmin(admin.options.InlineModelAdmin):
    ct_field = "content_type"
    ct_fk_field = "object_id"

    def get_content_types(self):
        ctypes = ContentType.objects.all().order_by('id').values_list('id', 'app_label', 'model')
        elements = {}
        for x, y, z in ctypes:
            try:
                elements[x] = reverse("admin:%s_%s_changelist" % (y, z))
            except NoReverseMatch:
                continue
        return json.dumps(elements)

    def get_formset(self, request, obj=None, **kwargs):
        result = super(GenericCollectionInlineModelAdmin, self).get_formset(request, obj, **kwargs)
        result.content_types = self.get_content_types()
        result.ct_fk_field = self.ct_fk_field
        return result

    class Media:
        js = ('contentrelations/js/genericlookup.js', )


class GenericCollectionTabularInline(GenericCollectionInlineModelAdmin):
    template = 'admin/edit_inline/gen_coll_tabular.html'


class GenericCollectionStackedInline(GenericCollectionInlineModelAdmin):
    template = 'admin/edit_inline/gen_coll_stacked.html'



from setuptools import setup, find_packages
import categories
import os

try:
    long_description = open('README.rst').read()
except IOError:
    long_description = ''

try:
    reqs = open(os.path.join(os.path.dirname(__file__), 'requirements.txt')).read()
except (IOError, OSError):
    reqs = ''

setup(
    name='django-categories',
    version=categories.get_version(),
    description='A way to handle one or more hierarchical category trees in django.',
    long_description=long_description,
    author='Corey Oordt',
    author_email='coreyoordt@gmail.com',
    include_package_data=True,
    url='http://github.com/callowayproject/django-categories',
    packages=find_packages(exclude=['example*', ]),
    classifiers=[
        'Framework :: Django',
    ],
    install_requires=reqs,
    dependency_links=[]
)



# -*- coding: utf-8 -*-


from django.db import models, migrations
import django.core.files.storage
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('slug', models.SlugField(verbose_name='slug')),
                ('active', models.BooleanField(default=True, verbose_name='active')),
                ('thumbnail', models.FileField(storage=django.core.files.storage.FileSystemStorage(), null=True, upload_to='uploads/categories/thumbnails', blank=True)),
                ('thumbnail_width', models.IntegerField(null=True, blank=True)),
                ('thumbnail_height', models.IntegerField(null=True, blank=True)),
                ('order', models.IntegerField(default=0)),
                ('alternate_title', models.CharField(default='', help_text='An alternative title to use on pages with this category.', max_length=100, blank=True)),
                ('alternate_url', models.CharField(help_text='An alternative URL to use instead of the one derived from the category hierarchy.', max_length=200, blank=True)),
                ('description', models.TextField(null=True, blank=True)),
                ('meta_keywords', models.CharField(default='', help_text='Comma-separated keywords for search engines.', max_length=255, blank=True)),
                ('meta_extra', models.TextField(default='', help_text='(Advanced) Any additional HTML to be placed verbatim in the &lt;head&gt;', blank=True)),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('parent', mptt.fields.TreeForeignKey(related_name='children', verbose_name='parent', blank=True, to='categories.Category', on_delete=models.CASCADE, null=True)),
            ],
            options={
                'ordering': ('tree_id', 'lft'),
                'abstract': False,
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='CategoryRelation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField(verbose_name='object id')),
                ('relation_type', models.CharField(help_text="A generic text field to tag a relation, like 'leadphoto'.", max_length='200', null=True, verbose_name='relation type', blank=True)),
                ('category', models.ForeignKey(verbose_name='category', to='categories.Category', on_delete=models.CASCADE)),
                ('content_type', models.ForeignKey(verbose_name='content type', to='contenttypes.ContentType', on_delete=models.CASCADE)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='category',
            unique_together=set([('parent', 'name')]),
        ),
    ]
